
#include "config.h"
#include "w5500/socket.h"
#include "util.h"
#include "w5500/w5500.h"
#include "device.h"
#include "dhcp.h"

#include <stdio.h> 
#include <string.h>


CONFIG_MSG  ConfigMsg, RecvMsg;

extern uint8 reboot_flag;

uint8 txsize[MAX_SOCK_NUM] = {2,2,2,2,2,2,2,2};
uint8 rxsize[MAX_SOCK_NUM] = {2,2,2,2,2,2,2,2};
//public buffer for DHCP, DNS, HTTP
uint8 BUFPUB[1460];
uint8 * strDHCPERR[]  = {"DHCP:NONE", "DHCP:ERR", "DHCP:TIMEOUT", "DHCP:UPDATE", "DHCP:CONFLICT", 0}; 

void write_config_to_eeprom(void)
{
  uint8 i, j;
  uint32 dAddr, data;
   
  // Unlock the Flash Program Erase controller
  FLASH_Unlock();
  // Clear All pending flags
  FLASH_ClearFlag(FLASH_FLAG_BSY | FLASH_FLAG_EOP | FLASH_FLAG_PGERR | FLASH_FLAG_WRPRTERR);	
  // Erase the FLASH pages 
  FLASH_ErasePage(ConfigAddr);
   
  dAddr = ConfigAddr;

  for (i = 0, j = 0; i < (uint8)(sizeof(ConfigMsg)/4);i++) 
  {
    data = *(u32 *)(ConfigMsg.mac+j);
    FLASH_ProgramWord(dAddr, data);
    dAddr += 4;
    j +=4;
  }
}


void Reset_W5500(void)
{
  GPIO_ResetBits(GPIOB, WIZ_RESET);
  Delay_us(2);  
  GPIO_SetBits(GPIOB, WIZ_RESET);
  Delay_ms(1500); 
}

void USART1_Init(void)
{
  USART_InitTypeDef USART_InitStructure;

  /* USARTx configuration ------------------------------------------------------*/
  /* USARTx configured as follow:
        - BaudRate = 115200 baud  
        - Word Length = 8 Bits
        - One Stop Bit
        - No parity
        - Hardware flow control disabled (RTS and CTS signals)
        - Receive and transmit enabled
  */
  USART_InitStructure.USART_BaudRate = 115200;
  USART_InitStructure.USART_WordLength = USART_WordLength_8b;
  USART_InitStructure.USART_StopBits = USART_StopBits_1;
  USART_InitStructure.USART_Parity = USART_Parity_No ;
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
  
  USART_ITConfig(USART1, USART_IT_RXNE, ENABLE); //Enable rx enable, 
  
  /* Configure the USARTx */ 
  USART_Init(USART1, &USART_InitStructure);
  /* Enable the USARTx */
  USART_Cmd(USART1, ENABLE);
}


//reboot 
void reboot(void)
{
  pFunction Jump_To_Application;
  uint32 JumpAddress;
  JumpAddress = *(vu32*) (0x00000004);
  Jump_To_Application = (pFunction) JumpAddress;
  Jump_To_Application();
}

void get_config(void)
{
  uint16 i;
  
  uint32 addr = ConfigAddr;
  for (i =0; i < CONFIG_MSG_SIZE; i++) 
  {
    *(ConfigMsg.mac+i) = *(vu8*)addr;
    addr += 1;
  }
}

void set_network(void)
{
  uint8 ip[4];
  uint8 mac[6]={0x00,0x08,0xdc,0x11,0x11,0x11};
  uint8 lip[4]={192,168,10,102};
  uint8 sub[4]={255,255,255,0};
  uint8 gw[4]={192,168,10,1};
  setSHAR(mac);
  setSUBR(sub);
  setGAR(gw);
  setSIPR(lip);

  //Init. TX & RX Memory size of w5200
  sysinit(txsize, rxsize); 
  
  setRTR(2000);//200ms
  setRCR(3);
  
  ConfigMsg.debug=1;
  getSIPR (ip);
  printf("IP : %d.%d.%d.%d\r\n", ip[0],ip[1],ip[2],ip[3]);

  getSUBR(ip);
  printf("SN : %d.%d.%d.%d\r\n", ip[0],ip[1],ip[2],ip[3]);

  getGAR(ip);
  printf("GW : %d.%d.%d.%d\r\n", ip[0],ip[1],ip[2],ip[3]);

}
