#include "config.h"
#include "socket.h"
#include "util.h"
#include "w5500.h"
#include "device.h"
#include "dhcp.h"
#include "dns.h"

#include "sockutil.h"

#include "at24c16.h"
#include "iap.h"

#include <stdio.h> 
#include <string.h>
#include <stdlib.h>

extern uint8 reboot_flag; //defined in main.c

uint8 DEFAULT_MAC[6] = {0x00, 0x08, 0xdc, 0x11, 0x12, 0x13};
uint8 DEFAULT_LIP[4] = {192, 168, 10, 112};
uint8 DEFAULT_SN[4] = {255, 255, 255, 0};
uint8 DEFAULT_GW[4] = {192, 168, 10, 1};
uint8 DEFAULT_DNS[4] = {8,8,8,8};
uint8 DEFAULT_RIP[4]={192,168,11,101};
uint8 DEFAULT_LOCATE_SIP[4]={192,168,11,111};
uint16 DEFAULT_LPORT=5000;
uint16 DEFAULT_RPORT=5001;
uint16 DEFAULT_LOCPORT=5005;

uint16 DEFAULT_KA_PERIOD=10; //unit: second
uint16 DEFAULT_KA_INTERVAL=10; //unit: second
uint16 DEFAULT_KA_COUNT=100;

uint16 DEFAULT_NAGLE=0;//unit: ms
uint16 DEFAULT_INACTIVITY=0;//unit: second
uint16 DEFAULT_RECONN=1000; //unit: ms

CONFIG_MSG  ConfigMsg, RecvMsg;

uint16 any_port=3000;

uint8 txsize[MAX_SOCK_NUM] = {2,2,2,2,2,2,2,2};
uint8 rxsize[MAX_SOCK_NUM] = {2,2,2,2,2,2,2,2};
//public buffer for DHCP, DNS, HTTP
uint8 BUFPUB[2048];
uint8 do_dns_flag=1;//do dns run flag
uint8 dns_retry_cnt=0;//dns retry counter
uint8 PUB_DOMAIN_BUF[64];//public domain buffer for main loop thread, max. length is 64 bytes 
//

extern char* buf;
extern uint8 trigger_code[3];
extern uint32 totalseconds;
extern uint16 ka_period_val;
extern uint16 ka_interval_val;
extern uint16 ka_count_val;

extern uint16 lport_val;

extern uint16 locport_val;

extern vu16 ka_interval;//unit: s


//
void set_default(void)
{
  memcpy(ConfigMsg.mac, DEFAULT_MAC, 6);
  memcpy(ConfigMsg.lip, DEFAULT_LIP, 4);
  memcpy(ConfigMsg.sub, DEFAULT_SN, 4);
  memcpy(ConfigMsg.gw,  DEFAULT_GW, 4);
  memcpy(ConfigMsg.dns, DEFAULT_DNS, 4);
  
  ConfigMsg.dhcp = STATIC_MODE;
  ConfigMsg.sw_ver[1]=FW_VER_LOW;
  ConfigMsg.sw_ver[0]=FW_VER_HIGH;
  ConfigMsg.debug = 1; 

  sprintf((char*)ConfigMsg.device_name,"%s","iWeb");
  sprintf((char*)ConfigMsg.device_serial,"iWeb-%02x%02x%02x%02x",ConfigMsg.mac[2],ConfigMsg.mac[3],ConfigMsg.mac[4],ConfigMsg.mac[5]);  
}

void write_config_to_eeprom(void)
{
  uint8 data;
  uint16 i,j;
  uint16 dAddr=0;
  for (i = 0, j = 0; i < (uint8)(sizeof(ConfigMsg)-4);i++) 
  {
    data = *(uint8 *)(ConfigMsg.mac+j);
    at24c16_write(dAddr, data);
    dAddr += 1;
    j +=1;
  }
}

void Reset_W5500(void)
{
  GPIO_ResetBits(GPIOB, WIZ_RESET);
  Delay_us(2);  
  GPIO_SetBits(GPIOB, WIZ_RESET);
  Delay_ms(1600); 
}


//reboot app
void reboot_app(void)
{
  //if using this function, the wwdg should be disabled
  pFunction Jump_To_Application;
  uint32 JumpAddress;
  JumpAddress = *(vu32*) (ApplicationAddress + 0x00000004);
  Jump_To_Application = (pFunction) JumpAddress;
  //initialize user application's stack pointer
  __set_MSP(*(__IO uint32_t *)ApplicationAddress);
  Jump_To_Application();
}
//reboot 
void reboot(void)
{
  pFunction Jump_To_Application;
  uint32 JumpAddress;
  JumpAddress = *(vu32*) (0x00000004);
  printf("%08x",JumpAddress);
  Jump_To_Application = (pFunction) JumpAddress;
  Jump_To_Application();
}
void get_config(void)
{
  uint8 tmp=0;
  for (uint16 i =0; i < CONFIG_MSG_LEN; i++) 
  {
    tmp=at24c16_read(i);
    *(ConfigMsg.mac+i) = tmp;
  }
  if((ConfigMsg.mac[0]==0xff)&&(ConfigMsg.mac[1]==0xff)&&(ConfigMsg.mac[2]==0xff)&&(ConfigMsg.mac[3]==0xff)&&(ConfigMsg.mac[4]==0xff)&&(ConfigMsg.mac[5]==0xff))
  {
    set_default();
    write_config_to_eeprom();
  }
  else
  {
    if((ConfigMsg.sw_ver[0]!=FW_VER_HIGH)||(ConfigMsg.sw_ver[1]!=FW_VER_LOW))//豸汾Ϣ
    {
      ConfigMsg.sw_ver[1] = FW_VER_LOW;
      ConfigMsg.sw_ver[0] = FW_VER_HIGH;
      write_config_to_eeprom();
    }
  }
}
void set_network(void)
{
  uint8 ip[4];
  setSHAR(ConfigMsg.mac);
  setSUBR(ConfigMsg.sub);
  setGAR(ConfigMsg.gw);
  setSIPR(ConfigMsg.lip);

  //Init. TX & RX Memory size of w5200
  sysinit(txsize, rxsize); 
  
  setRTR(2000);//200ms
  setRCR(3);

  getSIPR (ip);
  if (ConfigMsg.debug) 	
  {
    printf("IP : %d.%d.%d.%d:%d.%d.%d.%d\r\n", ip[0],ip[1],ip[2],ip[3],ConfigMsg.lip[0],ConfigMsg.lip[1],ConfigMsg.lip[2],ConfigMsg.lip[3]);
  }
  getSUBR(ip);
  if (ConfigMsg.debug) 	
  {
    printf("SN : %d.%d.%d.%d\r\n", ip[0],ip[1],ip[2],ip[3]);
  }
  getGAR(ip);
  if (ConfigMsg.debug) 	
  {
    printf("GW : %d.%d.%d.%d\r\n", ip[0],ip[1],ip[2],ip[3]);
  }
}

void do_udp_config(void)
{
  uint8 sckState;
  uint16 len;
  uint8 rIP[4];
  //0104
  uint8 broadcastIp[4]={255,255,255,255};
  uint16 rPort;
  //clear RecvMsg
  uint16 msg_len=sizeof(RecvMsg);
  memset(RecvMsg.op, 0, msg_len);
  
  sckState=getSn_SR(SOCK_CONFIG);
  switch(sckState)
  {
  case SOCK_UDP:
    if((len=getSn_RX_RSR(SOCK_CONFIG))>0)
    {
      len=recvfrom(SOCK_CONFIG, (uint8 *)&BUFPUB, len, rIP, &rPort);
      memcpy((uint8 *)&RecvMsg,(uint8 *)&BUFPUB,sizeof(RecvMsg));
      //FIND: searching, SETT: setting, FACT: factory reset, FIRS: firmware uploading
      if((RecvMsg.op[0]=='F') && (RecvMsg.op[1]=='I') && (RecvMsg.op[2]=='N') && (RecvMsg.op[3]=='D'))
      {

        RecvMsg = ConfigMsg;
        RecvMsg.op[0]='F';RecvMsg.op[1]='I';RecvMsg.op[2]='N';RecvMsg.op[3]='D';
        //response
        sendto(SOCK_CONFIG, (const uint8 *)RecvMsg.op, sizeof(RecvMsg), broadcastIp, rPort);
        
        if(ConfigMsg.debug) printf("FIND from %d.%d.%d.%d\r\n", rIP[0],rIP[1],rIP[2],rIP[3]);
      }
      else if((RecvMsg.op[0]=='S') && (RecvMsg.op[1]=='E') && (RecvMsg.op[2]=='T') && (RecvMsg.op[3]=='T'))
      {
        if((RecvMsg.mac[0]==ConfigMsg.mac[0]) && (RecvMsg.mac[1]==ConfigMsg.mac[1]) && (RecvMsg.mac[2]==ConfigMsg.mac[2]) && 
           (RecvMsg.mac[3]==ConfigMsg.mac[3]) && (RecvMsg.mac[4]==ConfigMsg.mac[4]) && (RecvMsg.mac[5]==ConfigMsg.mac[5]))
        {
          ConfigMsg = RecvMsg;//update config msg
          //version is fixed
          ConfigMsg.sw_ver[1] = FW_VER_LOW;
          ConfigMsg.sw_ver[0] = FW_VER_HIGH;
          ConfigMsg.state = FW_APP_NORMAL;
          write_config_to_eeprom();//write to flash
          //response
          sendto(SOCK_CONFIG, (uint8 *)&ConfigMsg, sizeof(ConfigMsg), broadcastIp, rPort);

          //reboot
          reboot_flag = 1;
        }
      }
      else if((RecvMsg.op[0]=='F') && (RecvMsg.op[1]=='I') && (RecvMsg.op[2]=='R') && (RecvMsg.op[3]=='M'))
      {
        if((RecvMsg.mac[0]==ConfigMsg.mac[0]) && (RecvMsg.mac[1]==ConfigMsg.mac[1]) && (RecvMsg.mac[2]==ConfigMsg.mac[2]) && 
           (RecvMsg.mac[3]==ConfigMsg.mac[3]) && (RecvMsg.mac[4]==ConfigMsg.mac[4]) && (RecvMsg.mac[5]==ConfigMsg.mac[5]))
        {
          //set module state
          ConfigMsg.state = FW_UPDATE_CONFIG;
          //response, send FIRM to config tool.
          sendto(SOCK_CONFIG, (uint8 *)&RecvMsg, sizeof(ConfigMsg), broadcastIp, rPort);
          if(ConfigMsg.debug) printf("Firware will be updated by Configtool!\r\n");
        }
      }
    }
    break;
  case SOCK_CLOSED:
    socket(SOCK_CONFIG, Sn_MR_UDP, REMOTE_CLIENT_PORT, 0);
    //printf("UDP configuation ready\r\n");
    break;
  }
}


uint32 rxLen=0;
uint32 fLen=0;
uint32 nPage=0;
uint32 nErasedPage=0;
uint32 flashDest =ApplicationBackAddress;
uint16 pageSize=0x400;
uint8 fw_buf[1024]={0x00,};
uint8 isErased=0;
uint32 calc_checksum=0;

#define DEBUG_UPDATE

void do_fw_update(void)
{
  if((ConfigMsg.state != FW_UPDATE_CONFIG)&&(ConfigMsg.state != WEBPAGE_UPDATE_CONFIG)) return;
  
  uint32 len=0;
  uint32 i=0;
  
  switch(getSn_SR(SOCK_FW))
  {
  case SOCK_ESTABLISHED:
    if(getSn_IR(SOCK_FW) & Sn_IR_CON)
    {
      rxLen=0;
      nPage=0;
      nErasedPage=0;
      fLen=0;
      isErased=0;

#ifdef DEBUG_UPDATE
      printf("unlock flash\r\n");
#endif
      //unlock flash
      FLASH_Unlock();
      // Clear All pending flags
      FLASH_ClearFlag(FLASH_FLAG_BSY | FLASH_FLAG_EOP | FLASH_FLAG_PGERR | FLASH_FLAG_WRPRTERR);
      setSn_IR(SOCK_FW, Sn_IR_CON);
      if(ConfigMsg.state == FW_UPDATE_CONFIG)
      {
        //clear fw length and checksum in ConfigMsg
        ConfigMsg.fw_len = 0;
        calc_checksum = 0;
      }
#ifdef DEBUG_UPDATE
      printf("connected\r\n");
#endif
    }
    //some timeout function should be added here
    if((len=getSn_RX_RSR(SOCK_FW))>0)
    {
      if((len==4) && (isErased==0))
      {
        recv(SOCK_FW, (uint8*)&fLen,4);
#ifdef DEBUG_UPDATE
        printf("file len=%d\r\n",fLen);
#endif
        if(ConfigMsg.state == FW_UPDATE_CONFIG)
        {
          nPage=FLASH_PagesMask(fLen);
        }
        else
          nPage=32; //webpage space is 32KB, 
#ifdef DEBUG_UPDATE
        printf("page=%d\r\n",nPage);
#endif
        __disable_irq();
        for(nErasedPage=0; nErasedPage<nPage; nErasedPage++)
        {
          IWDG_ReloadCounter();//feed iwdg
          FLASH_ErasePage(flashDest + pageSize*nErasedPage);
        }
#ifdef DEBUG_UPDATE
        printf("erase over\r\n");
#endif
        __enable_irq();
        isErased=1;
        //send the len to PC program to tell him flash erased over
        send(SOCK_FW, (uint8*)&fLen, 4);
        
        //update the fw len to ConfigMsg
        ConfigMsg.fw_len = fLen;
        
        if(ConfigMsg.debug) printf(">");
      }
      else
      {
        recv(SOCK_FW, fw_buf, len);
        if(ConfigMsg.debug) printf(".");
        for(i=0;i<len;i+=4)
        {
          FLASH_ProgramWord(flashDest,*(uint32*)((uint32)fw_buf + i));
          
          //calcute fw checksum for ConfigMsg
          //ConfigMsg.fw_checksum += *(uint32*)((uint32)fw_buf + i);
          if(ConfigMsg.state == FW_UPDATE_CONFIG) calc_checksum += *(uint32*)(flashDest);
          flashDest+=4;
        }
        
        rxLen+=len;
        //echo to PC program
        send(SOCK_FW, (uint8*)&len, (uint16)4);
        
        if(rxLen==fLen)
        {
          disconnect(SOCK_FW);
          
          //save to flash
          if(ConfigMsg.state == FW_UPDATE_CONFIG)
          {
            //save fw checksum and module state
            ConfigMsg.state = FW_RXD_HTTP;
            ConfigMsg.sw_ver[0] = FW_VER_HIGH;
            ConfigMsg.sw_ver[1] = FW_VER_LOW;
            ConfigMsg.fw_checksum = calc_checksum;
#ifdef DEBUG_UPDATE
            printf("OK.%d\r\n", calc_checksum);
#endif
#ifdef DEBUG_UPDATE
            printf("ver=%d.%d\r\n", FW_VER_HIGH, FW_VER_LOW);
#endif
            
            printf("write eeprom\r\n");
            write_config_to_eeprom();
            printf("write eeprom OK\r\n");

          
            //jump to app
            //for(i=0;i<1000;i++);//wait for a while
            reboot_flag = 1;
            printf("reboot\r\n");
          }
          else
            FLASH_ProgramWord(flashDest,0x00000000);
          
          //lock flash again
          FLASH_Lock();
        }
      }
    }
    break;
  case SOCK_CLOSE_WAIT:
    //disconnect(SOCK_FW);
    break;
  case SOCK_CLOSED:
    
    close(SOCK_FW);  
    socket(SOCK_FW,Sn_MR_TCP,30000,Sn_MR_ND);
    break;
  case SOCK_INIT:
    
    listen(SOCK_FW);
    break;
  }
}
void display_netpara(void)
{
  printf("IP Addres: %d.%d.%d.%d\r\n", (uint8)ConfigMsg.lip[0],(uint8)ConfigMsg.lip[1],(uint8)ConfigMsg.lip[2],(uint8)ConfigMsg.lip[3]);
  printf("Subnet mask: %d.%d.%d.%d\r\n", (uint8)ConfigMsg.sub[0],(uint8)ConfigMsg.sub[1],(uint8)ConfigMsg.sub[2],(uint8)ConfigMsg.sub[3]);
  printf("Gateway: %d.%d.%d.%d\r\n", (uint8)ConfigMsg.gw[0],(uint8)ConfigMsg.gw[1],(uint8)ConfigMsg.gw[2],(uint8)ConfigMsg.gw[3]);
  printf("DNS server: %d.%d.%d.%d\r\n", (uint8)ConfigMsg.dns[0],(uint8)ConfigMsg.dns[1],(uint8)ConfigMsg.dns[2],(uint8)ConfigMsg.dns[3]);
}

//build io status json string
void json_io_status(int8* json)
{
  sprintf(json, "IoStatusCallback({\"din\":[{\"v\":\"%d\"},\
          {\"v\":\"%d\"},\
          {\"v\":\"%d\"},\
          {\"v\":\"%d\"}],\
          \"dout\":[{\"v\":\"%d\"},\
          {\"v\":\"%d\"},\
          {\"v\":\"%d\"},\
          {\"v\":\"%d\"}]});",
          GPIO_ReadInputDataBit(GPIOA,KEY1),
          GPIO_ReadInputDataBit(GPIOA,KEY2),
          GPIO_ReadInputDataBit(GPIOA,KEY3),
          GPIO_ReadInputDataBit(GPIOA,KEY4),
          GPIO_ReadInputDataBit(GPIOA,LED1),
          GPIO_ReadInputDataBit(GPIOA,LED2),
          GPIO_ReadInputDataBit(GPIOA,LED3),
          GPIO_ReadInputDataBit(GPIOA,LED4));

}
//digital out set by pin
void set_dout(uint8 pin, uint8 val)
{
  switch (pin)
  {
  case 0:
    if(val==1)
    {
      GPIO_SetBits(GPIOA, LED1);
    }
    else 
    {
      GPIO_ResetBits(GPIOA, LED1);
    }
    break;
  case 1:
    if(val==1)
    {
      GPIO_SetBits(GPIOA, LED2);
    }
    else 
    {
      GPIO_ResetBits(GPIOA, LED2);
    }
    break;
  case 2:
    if(val==1)
    {
      GPIO_SetBits(GPIOA, LED3);
    }
    else 
    {
      GPIO_ResetBits(GPIOA, LED3);
    }
    break;
  case 3:
    if(val==1)
    {
      GPIO_SetBits(GPIOA, LED4);
    }
    else 
    {
      GPIO_ResetBits(GPIOA, LED4);
    }
    break;
  default:
    break;
  }
}
//set digital output by 8-bit value
void d_output(uint8 val)
{
  uint8 i;
  uint8 tmp=1;
  for(i=0;i<4;i++)
  {
    tmp=1<<i;
    if(tmp==(val&tmp))
      set_dout(i,1);
    else
      set_dout(i,0);
    
  }
}
//get digital input
uint8 get_din(uint8 pin)
{
  uint8 ret=0;
  switch(pin)
  {
  case 0:
    ret = GPIO_ReadInputDataBit(GPIOA, KEY1);
    break;
  case 1:
    ret = GPIO_ReadInputDataBit(GPIOA, KEY2);
    break;
  case 2:
    ret = GPIO_ReadInputDataBit(GPIOA, KEY3);
    break;
  case 3:
    ret = GPIO_ReadInputDataBit(GPIOA, KEY4);
    break;
  default:
    break;
  }
  return (ret);
}