#ifndef _DEVICE_H_
#define _DEVICE_H_


#define FW_VER_HIGH   2
#define FW_VER_LOW    1

#define BR_115200	0xFF
#define BR_57600	0xFE
#define BR_38400	0xFD
#define BR_19200	0xFA
#define BR_9600		0xF4
#define BR_4800		0xE8
#define BR_2400		0xD0
#define BR_1200		0xA0

#define BR_115200_INDEX     7
#define BR_57600_INDEX      6
#define BR_38400_INDEX      5
#define BR_19200_INDEX      4
#define BR_9600_INDEX       3
#define BR_4800_INDEX       2
#define BR_2400_INDEX       1
#define BR_1200_INDEX       0


#define DATABIT_7	7
#define DATABIT_8	8

#define DATABIT_7_INDEX 0
#define DATABIT_8_INDEX 1

#define PARITY_NONE	0
#define PARITY_ODD	1
#define PARITY_EVEN	2

#define STOPBIT_1	1
#define STOPBIT_1_INDEX 0

#define FLOW_NONE       0
#define FLOW_RTSCTS	1

#define TCP_CLIENT_MODE           0
#define TCP_SERVER_MODE           1
#define UDP_MODE                  2
#define TCP_MIXED_MODE 	          3



/* UDP port numbers for Remote Configuration */
#define REMOTE_SERVER_PORT	5005	/* from server to client */
#define REMOTE_CLIENT_PORT	1460	/* from client to server */
#define REMOTE_CLIENT_PORT1	1461	/* from client to server */
#define REMOTE_UPDATE_PORT	1470	/* from client to server */

#define REMOTE_FIND 	0
#define REMOTE_SETT 	1
#define REMOTE_SMAC 	2
#define REMOTE_FW 	3

typedef  void (*pFunction)(void);

#define STATIC_MODE 		0
#define DHCP_MODE 		1

#define CONFIG_MSG_SIZE		sizeof(CONFIG_MSG)		

#define EE_OP  0  // Operation Mode

#define EE_MAC1  0 // MAC
#define EE_MAC2  1
#define EE_MAC3  2
#define EE_MAC4  3
#define EE_MAC5  4
#define EE_MAC6  5

#define EE_VER_H  6  // SW_Version
#define EE_VER_L  7  // SW_Version

#define EE_DEVICE_NAME   8 //16bytes
#define EE_DEVICE_SERIAL 24//device serial number; 16bytes

#define EE_LIP1   40  // IP addr
#define EE_LIP2   41
#define EE_LIP3   42
#define EE_LIP4   43

#define EE_SUB1   44  // Subnet mask
#define EE_SUB2   45
#define EE_SUB3   46
#define EE_SUB4   47

#define EE_GW1   48 // Gateway addr
#define EE_GW2   49
#define EE_GW3   50
#define EE_GW4   51
#define EE_DNS1 52
#define EE_DNS2 53 
#define EE_DNS3 54
#define EE_DNS4 55

#define EE_DHCP   56 // DHCP 1 0n, 0 0ff

#define EE_DEBUG  57  // Debug code

/* Serial Info */
#define EE_BAUD    58  // serial baudrate
#define EE_DSIZE   59  // Data size : 8, 7
#define EE_PARITY  60  // Parity : none, even, odd
#define EE_STOP    61  // Stop bit : 1
#define EE_FLOW    62  // Flow control : None, RTS/CTS

#define EE_PWD     63 //16 bytes MD5 digest

#define EE_MODE  	79  // tcp client 0, tcp server mode 1, tcp mixed 2, udp mode 3
#define EE_LPORTH 	80
#define EE_LPORTL 	81

#define EE_RIP1  82 // Server IP (if EE_KIND == client)
#define EE_RIP2  83
#define EE_RIP3  84
#define EE_RIP4  85

#define EE_RPORTH 86 // Server Port Number
#define EE_RPORTL 87

#define EE_DNS_FLAG      88

#define EE_DOMAIN       89 //32 bytes

//NTP
#define EE_NTP_FLAG     121
#define EE_NTP_SERVER_IP   122 //NTP server domain name, 32bytes-->4bytes
#define EE_TIME_ZONE    126 //time zone


/*
#define EE_UDP   40 //

#define EE_DNS_FLAG  41

#define EE_DOMAIN 	 42  // 32 byte   51+32=83-1

//locating server info
#define EE_LOC_ENABLE  74
#define EE_LOC_IP1     75
#define EE_LOC_IP2     76
#define EE_LOC_IP3     77
#define EE_LOC_IP4     78

#define EE_LOC_PORTH   79
#define EE_LOC_PORTL   80

//keep alive
#define EE_KA_FLAG     81
#define EE_KA_PERIODH  82//if no reply for the period, then send KA (second)
#define EE_KA_PERIODL  83
#define EE_KA_COUNTH   84//how many times to send KA
#define EE_KA_COUNTL   85
#define EE_KA_INTERVALH 86//Sending KA interval (second)
#define EE_KA_INTERVALL 87

#define EE_PWD          88 //16 bytes, md5 digest

#define EE_NTP_DOMAIN   104 //NTP server domain name, 32bytes-->4bytes
#define EE_TIME_ZONE    108 //time zone
#define EE_NTP_FLAG     109
#define EE_USER_DATE    110 //user input date time, 7bytes
#define EE_STATE        111 //module status
#define EE_NAGLE        112 //timer option, 2bytes
#define EE_INACT        114 //inactivity, 2bytes
#define EE_RECONN       116 //reconnection, 2bytes
#define EE_DEVICE_NAME   117 //16bytes
#define EE_DEVICE_SERIAL EE_DEVICE_NAME+16//device serial number; 16bytes
*/

extern uint8 BUFPUB[2048];
extern uint8 PUB_DOMAIN_BUF[64];//public domain buffer
#define gBUFPUBLIC BUFPUB;
#define DEVICE_ID "iWeb"

#define DEFAULT_HTTP_PORT			80
#define NET_CONFIG_CGI "config.pl"
#define NET_PWD_CGI    "password.pl"

void set_default(void);
int8 atonum(int8 ch);
void set_network(void);
uint8 get_hex(uint8 b0, uint8 b1);
void write_config_to_eeprom(void);

void Reset_W5500(void);


void reboot_app(void);
void reboot(void);

void do_fw_update(void);

void get_config(void);
void do_udp_config(void);
void do_dhcp(void);
void do_dns(void);

void write_change_flag(uint8 flg);
uint8 read_change_flag(void);


//json
void json_io_status(int8*);
//digital out set
void set_dout(uint8 pin, uint8 val);
void d_output(uint8 val);
#endif