/**
  ******************************************************************************
  * @file    Project/STM32F10x_StdPeriph_Template/main.c 
  * @author  MCD Application Team
  * @version V3.5.0
  * @date    08-April-2011
  * @brief   Main program body
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2011 STMicroelectronics</center></h2>
  ******************************************************************************
  */  

/* Includes ------------------------------------------------------------------*/
#include "config.h"
#include "mcu_init.h"
#include "device.h"
#include "spi2.h"
#include "util.h"
#include "httputil.h"
#include "base64.h"
//#include "weibo.h"
#include "at24c16.h"
#include "stm32f10x_wwdg.h"

#include <stdio.h>




/** @addtogroup STM32F10x_StdPeriph_Template
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/

/* Private macro -------------------------------------------------------------*/
#ifdef __GNUC__
  /* With GCC/RAISONANCE, small printf (option LD Linker->Libraries->Small printf
     set to 'Yes') calls __io_putchar() */
  #define PUTCHAR_PROTOTYPE int __io_putchar(int ch)
#else
  #define PUTCHAR_PROTOTYPE int fputc(int ch, FILE *f)
#endif /* __GNUC__ */
/* Private variables ---------------------------------------------------------*/
vu32 ms=0;
vu8 sec=0,min=0;
vu8 hour=0;

//keep alive
vu16 ka_period=0; //unit: s
vu16 ka_interval=0;//unit: s
//
vu8 http_time_flag = 0;
vu16 http_time_cnt = 0;//after 5 minutes, the pwd_rxd flag will be reset.

uint8 reboot_flag = 0;

/**
  * @brief  Main program.
  * @param  None
  * @retval None
*/

int main(void)
{  
  RCC_Configuration();
  GPIO_Configuration();
  NVIC_Configuration();
  Timer_Configuration();
  at24c16_init();
  USART1_Init();

  Reset_W5500(); 
  WIZ_SPI_Init();
  
  get_config();//read config data
  
  printf("Firmware ver%d.%d\r\n",ConfigMsg.sw_ver[0],ConfigMsg.sw_ver[1]);

  if(ConfigMsg.debug==0) ConfigMsg.debug=1;
  
  /**********************************************/
  set_network();
  /* Infinite loop */
  while (1)
  {
    do_http(SOCK_HTTP);  
    if(reboot_flag) reboot();   
  }
}

/**************************/
/*****Timer2 interrupt*****/
/**************************/
void Timer2_ISR(void)
{ 
  ms++;//mili second  
  if(ms>=1000)//second
  {  
    ms=0;
    sec++;
    if(http_time_flag) 
      http_time_cnt++;

  }
  if(sec>=60)//minute
  {
    sec=0;
    min++;
  }
  if(min>=60)//hour
  {
    min=0;
    hour++;
  }
  if(hour>=24)//day
  {
    hour=0;
  }
}

#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif

/**
  * @}
  */


/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/
